import { __decorate } from 'tslib';
import { Injectable, InjectionToken, NgModule } from '@angular/core';
import { HTTP_INTERCEPTORS } from '@angular/common/http';
import { BLOCKUI_DEFAULT, BlockUIService, BlockUIModule } from 'ng-block-ui';
import { finalize } from 'rxjs/operators';

let BlockUIHttpSettings = class BlockUIHttpSettings {
    constructor() {
        this.settings = {};
    }
};
BlockUIHttpSettings = __decorate([
    Injectable()
], BlockUIHttpSettings);

let BlockUIInterceptor = class BlockUIInterceptor {
    constructor(blockUIService, blockUIHttpSettings) {
        this.blockUIService = blockUIService;
        this.blockUIHttpSettings = blockUIHttpSettings;
        this.activeHttpRequests = 0;
    }
    intercept(request, next) {
        let active = false;
        if (this.shouldBlock(request)) {
            this.blockUIService.start(BLOCKUI_DEFAULT);
        }
        return next.handle(request)
            .pipe(finalize(() => {
            if (this.shouldBlock(request)) {
                const { blockAllRequestsInProgress } = this.blockUIHttpSettings.settings;
                const method = blockAllRequestsInProgress ? 'stop' : 'reset';
                this.blockUIService[method](BLOCKUI_DEFAULT);
            }
        }));
    }
    shouldBlock(request) {
        const { method, urlWithParams } = request;
        const { settings } = this.blockUIHttpSettings;
        const requestFilters = settings.requestFilters || [];
        return !requestFilters.some((f) => {
            if (f && f.method && f.url) {
                return f.method.toUpperCase() === method && f.url.test(urlWithParams);
            }
            else if (typeof f === 'function') {
                return f(request);
            }
            return f.test(urlWithParams);
        });
    }
};
BlockUIInterceptor.ctorParameters = () => [
    { type: BlockUIService },
    { type: BlockUIHttpSettings }
];
BlockUIInterceptor = __decorate([
    Injectable()
], BlockUIInterceptor);

var BlockUIHttpModule_1;
// Needed for AOT compiling
const BlockUIHttpModuleSettings = new InjectionToken('BlockUIHttpModuleSettings');
function provideSettingsInstance(settings) {
    return {
        settings: Object.assign({ blockAllRequestsInProgress: true }, settings)
    };
}
let BlockUIHttpModule = BlockUIHttpModule_1 = class BlockUIHttpModule {
    static forRoot(settings = {}) {
        return {
            ngModule: BlockUIHttpModule_1,
            providers: [
                {
                    provide: BlockUIHttpModuleSettings,
                    useValue: settings
                },
                {
                    provide: BlockUIHttpSettings,
                    useFactory: provideSettingsInstance,
                    deps: [BlockUIHttpModuleSettings]
                },
                {
                    provide: HTTP_INTERCEPTORS,
                    useClass: BlockUIInterceptor,
                    multi: true
                }
            ]
        };
    }
};
BlockUIHttpModule = BlockUIHttpModule_1 = __decorate([
    NgModule({
        imports: [BlockUIModule]
    })
], BlockUIHttpModule);

/**
 * Generated bundle index. Do not edit.
 */

export { BlockUIHttpModule, BlockUIHttpModuleSettings as ɵa, provideSettingsInstance as ɵb, BlockUIHttpSettings as ɵc, BlockUIInterceptor as ɵd };
//# sourceMappingURL=ng-block-ui-http.js.map
